require("parallel")

## number of paths to simulate
n <- 400000

## pricing European Options (sequential)

MC_sim <- function(sigma, r, S, T, X, n_paths = 10^5){
  price <- double(n_paths)
  for( i in 1:n_paths){
      price[i] <- exp(-r * T) * max(S * exp((r - 1/2*sigma^2)*T + sigma*sqrt(T)*rnorm(1)) - X, 0)
  }

  list( v = mean(price), ci = c(mean(price)-1.96*sd(price)/sqrt(n_paths),
                               mean(price)+1.96*sd(price)/sqrt(n_paths)) )
}

## parallel
MC_sim_par <- function(cl, sigma, r, S, T, X, n_per_node = 10000, nodes = 1){
    foo <- function(n, sigma, r, S, T, X) {
      price <- double(n)
      for(i in 1:n)
        price[i] <- exp(-r * T) * max(S * exp((r - 1/2*sigma^2)*T + sigma*sqrt(T)*rnorm(1)) - X, 0)
      price
    }
    price <- unlist(parLapply(cl, rep(n_per_node, nodes), foo, sigma, r, S, T, X))


    list( v = mean(price), ci = c(mean(price)-1.96*sd(price)/sqrt(n_per_node * nodes),
                           mean(price)+1.96*sd(price)/sqrt(n_per_node * nodes)) )
}


slots <- as.integer(Sys.getenv("NSLOTS"))

## start MPI cluster and retrieve the nodes we are working on.
cl <- makeCluster(slots, type = "PSOCK")
clusterCall(cl, function() Sys.info()[c("nodename","machine")])

## note that this must be an integer
sim_per_slot <- as.integer(n/slots)

## setup PRNG
clusterSetRNGStream(cl, iseed = 123)
price <- MC_sim_par(cl, sigma = 0.2, S = 120, T = 1, X = 130, r = 0.05, n_per_node = sim_per_slot, nodes = slots)
price

stopCluster(cl)
